@@ -1,4 +1,4 @@
-__version__ = '0.5.5'
+__version__ = '0.5.7'
 
 
 # amalgamate exclude jupyter_kernel parser_table parser_test_table pyghooks
@@ -18,14 +18,16 @@ else:
         _sys.modules['xonsh.lazyasd'] = __amalgam__
         lazyjson = __amalgam__
         _sys.modules['xonsh.lazyjson'] = __amalgam__
-        ansi_colors = __amalgam__
-        _sys.modules['xonsh.ansi_colors'] = __amalgam__
-        codecache = __amalgam__
-        _sys.modules['xonsh.codecache'] = __amalgam__
+        color_tools = __amalgam__
+        _sys.modules['xonsh.color_tools'] = __amalgam__
         platform = __amalgam__
         _sys.modules['xonsh.platform'] = __amalgam__
         pretty = __amalgam__
         _sys.modules['xonsh.pretty'] = __amalgam__
+        ansi_colors = __amalgam__
+        _sys.modules['xonsh.ansi_colors'] = __amalgam__
+        codecache = __amalgam__
+        _sys.modules['xonsh.codecache'] = __amalgam__
         lazyimps = __amalgam__
         _sys.modules['xonsh.lazyimps'] = __amalgam__
         parser = __amalgam__
@@ -48,6 +50,8 @@ else:
         _sys.modules['xonsh.foreign_shells'] = __amalgam__
         jobs = __amalgam__
         _sys.modules['xonsh.jobs'] = __amalgam__
+        jsonutils = __amalgam__
+        _sys.modules['xonsh.jsonutils'] = __amalgam__
         lexer = __amalgam__
         _sys.modules['xonsh.lexer'] = __amalgam__
         openpy = __amalgam__
