@@ -199,10 +199,18 @@ def _SOURCE_FOREIGN_PARSER():
     parser.add_argument('--seterrpostcmd', default=None, dest='seterrpostcmd',
                         help='command(s) to set exit-on-error after all'
                              'other commands.')
+    parser.add_argument('--overwrite-aliases', default=False, action='store_true',
+                        dest='overwrite_aliases',
+                        help='flag for whether or not sourced aliases should '
+                             'replace the current xonsh aliases.')
+    parser.add_argument('--show', default=False, action='store_true', dest='show',
+                        help='Will show the script output.')
+    parser.add_argument('-d', '--dry-run', default=False, action='store_true',
+                        dest='dryrun', help='Will not actually source the file.')
     return parser
 
 
-def source_foreign(args, stdin=None):
+def source_foreign(args, stdin=None, stdout=None, stderr=None):
     """Sources a file written in a foreign shell language."""
     ns = _SOURCE_FOREIGN_PARSER.parse_args(args)
     if ns.prevcmd is not None:
@@ -224,10 +232,15 @@ def source_foreign(args, stdin=None):
                                           sourcer=ns.sourcer,
                                           use_tmpfile=ns.use_tmpfile,
                                           seterrprevcmd=ns.seterrprevcmd,
-                                          seterrpostcmd=ns.seterrpostcmd)
+                                          seterrpostcmd=ns.seterrpostcmd,
+                                          show=ns.show,
+                                          dryrun=ns.dryrun)
     if fsenv is None:
-        return (None, 'xonsh: error: Source failed: '
-                      '{}\n'.format(ns.prevcmd), 1)
+        if ns.dryrun:
+            return
+        else:
+            msg = 'xonsh: error: Source failed: {}\n'.format(ns.prevcmd)
+            return (None, msg, 1)
     # apply results
     env = builtins.__xonsh_env__
     denv = env.detype()
@@ -244,7 +257,13 @@ def source_foreign(args, stdin=None):
     for k, v in fsaliases.items():
         if k in baliases and v == baliases[k]:
             continue  # no change from original
-        baliases[k] = v
+        elif ns.overwrite_aliases or k not in baliases:
+            baliases[k] = v
+        else:
+            msg = ('Skipping application of {0!r} alias from {1!r} '
+                   'since it shares a name with an existing xonsh alias. '
+                   'Use "--overwrite-alias" option to apply it anyway.')
+            print(msg.format(k, ns.shell), file=stderr)
 
 
 def source_alias(args, stdin=None):
@@ -353,11 +372,12 @@ def xonfig(args, stdin=None):
 
 
 @unthreadable
-def trace(args, stdin=None):
+def trace(args, stdin=None, stdout=None, stderr=None, spec=None):
     """Runs the xonsh tracer utility."""
     from xonsh.tracer import tracermain  # lazy import
     try:
-        return tracermain(args)
+        return tracermain(args, stdin=stdin, stdout=stdout,
+                          stderr=stderr, spec=spec)
     except SystemExit:
         pass
 
