@@ -2,6 +2,7 @@
 """The xonsh abstract syntax tree node."""
 # These are imported into our module namespace for the benefit of parser.py.
 # pylint: disable=unused-import
+import sys
 from ast import (
     Module, Num, Expr, Str, Bytes, UnaryOp, UAdd, USub, Invert,
     BinOp, Add, Sub, Mult, Div, FloorDiv, Mod, Pow, Compare, Lt, Gt,
@@ -91,11 +92,25 @@ def min_col(node):
 def max_col(node):
     """Returns the maximum col_offset of the node and all sub-nodes."""
     col = getattr(node, 'max_col', None)
-    if col is None:
-        col = max(map(get_col, walk(node)))
+    if col is not None:
+        return col
+    highest = max(walk(node), key=get_col)
+    col = highest.col_offset + node_len(highest)
     return col
 
 
+def node_len(node):
+    """The length of a node as a string"""
+    val = 0
+    for n in walk(node):
+        if isinstance(n, Name):
+            val += len(n.id)
+        elif isinstance(n, Attribute):
+            val += 1 + (len(n.attr) if isinstance(n.attr, str) else 0)
+        # this may need to be added to for more nodes as more cases are found
+    return val
+
+
 def get_id(node, default=None):
     """Gets the id attribute of a node, or returns a default."""
     return getattr(node, 'id', default)
@@ -171,8 +186,10 @@ class CtxAwareTransformer(NodeTransformer):
         self.lines = None
         self.mode = None
         self._nwith = 0
+        self.filename = '<xonsh-code>'
+        self.debug_level = 0
 
-    def ctxvisit(self, node, inp, ctx, mode='exec'):
+    def ctxvisit(self, node, inp, ctx, mode='exec', filename=None, debug_level=0):
         """Transforms the node in a context-dependent way.
 
         Parameters
@@ -183,12 +200,18 @@ class CtxAwareTransformer(NodeTransformer):
             The input code in string format.
         ctx : dict
             The root context to use.
+        filename : str, optional
+            File we are to transform.
+        debug_level : int, optional
+            Debugging level to use in lexing and parsing.
 
         Returns
         -------
         node : ast.AST
             The transformed node.
         """
+        self.filename = self.filename if filename is None else filename
+        self.debug_level = debug_level
         self.lines = inp.splitlines()
         self.contexts = [ctx, set()]
         self.mode = mode
@@ -220,28 +243,41 @@ class CtxAwareTransformer(NodeTransformer):
             mincol = len(line) - len(line.lstrip())
             maxcol = None
         else:
-            mincol = min_col(node)
+            mincol = max(min_col(node) - 1, 0)
             maxcol = max_col(node)
             if mincol == maxcol:
                 maxcol = find_next_break(line, mincol=mincol,
                                          lexer=self.parser.lexer)
             else:
                 maxcol += 1
-        spline = subproc_toks(line,
-                              mincol=mincol,
-                              maxcol=maxcol,
-                              returnline=False,
-                              lexer=self.parser.lexer)
+        spline = subproc_toks(line, mincol=mincol, maxcol=maxcol,
+                              returnline=False, lexer=self.parser.lexer)
+        if spline is None or len(spline) < len(line[mincol:maxcol]) + 2:
+            # failed to get something consistent, try greedy wrap
+            # The +2 comes from "![]" being length 3, minus 1 since maxcol
+            # is one beyond the total length for slicing
+            spline = subproc_toks(line, mincol=mincol, maxcol=maxcol,
+                                  returnline=False, lexer=self.parser.lexer,
+                                  greedy=True)
         if spline is None:
             return node
         try:
-            newnode = self.parser.parse(spline, mode=self.mode)
+            newnode = self.parser.parse(spline, mode=self.mode,
+                                        filename=self.filename,
+                                        debug_level=(self.debug_level > 2))
             newnode = newnode.body
             if not isinstance(newnode, AST):
                 # take the first (and only) Expr
                 newnode = newnode[0]
             increment_lineno(newnode, n=node.lineno - 1)
             newnode.col_offset = node.col_offset
+            if self.debug_level > 1:
+                msg = ('{0}:{1}:{2}{3} - {4}\n'
+                       '{0}:{1}:{2}{3} + {5}')
+                mstr = '' if maxcol is None else ':' + str(maxcol)
+                msg = msg.format(self.filename, node.lineno,
+                                 mincol, mstr, line, spline)
+                print(msg, file=sys.stderr)
         except SyntaxError:
             newnode = node
         if strip_expr and isinstance(newnode, Expr):
@@ -262,85 +298,6 @@ class CtxAwareTransformer(NodeTransformer):
                 break
         return inscope
 
-    #
-    # With Transformers
-    #
-    def insert_with_block_check(self, node):
-        """Modifies a with statement node in-place to add an initial check
-        for whether or not the block should be executed. If the block is
-        not executed it will raise a XonshBlockError containing the required
-        information.
-        """
-        nwith = self._nwith  # the nesting level of the current with-statement
-        lineno = get_lineno(node)
-        col = get_col(node, 0)
-        # Add or discover target names
-        targets = set()
-        i = 0  # index of unassigned items
-
-        def make_next_target():
-            nonlocal i
-            targ = '__xonsh_with_target_{}_{}__'.format(nwith, i)
-            n = Name(id=targ, ctx=Store(), lineno=lineno, col_offset=col)
-            targets.add(targ)
-            i += 1
-            return n
-        for item in node.items:
-            if item.optional_vars is None:
-                if has_elts(item.context_expr):
-                    targs = [make_next_target() for _ in item.context_expr.elts]
-                    optvars = Tuple(elts=targs, ctx=Store(), lineno=lineno,
-                                    col_offset=col)
-                else:
-                    optvars = make_next_target()
-                item.optional_vars = optvars
-            else:
-                targets.update(gather_names(item.optional_vars))
-        # Ok, now that targets have been found / created, make the actual check
-        # to see if we are in a non-executing block. This is equivalent to
-        # writing the following condition:
-        #
-        #     if getattr(targ0, '__xonsh_block__', False) or \
-        #        getattr(targ1, '__xonsh_block__', False) or ...:
-        #         raise XonshBlockError(lines, globals(), locals())
-        tests = [_getblockattr(t, lineno, col) for t in sorted(targets)]
-        if len(tests) == 1:
-            test = tests[0]
-        else:
-            test = BoolOp(op=Or(), values=tests, lineno=lineno, col_offset=col)
-        ldx, udx = self._find_with_block_line_idx(node)
-        lines = [Str(s=s, lineno=lineno, col_offset=col)
-                 for s in self.lines[ldx:udx]]
-        check = If(test=test, body=[
-            Raise(exc=xonsh_call('XonshBlockError',
-                                 args=[List(elts=lines, ctx=Load(),
-                                            lineno=lineno, col_offset=col),
-                                       xonsh_call('globals', args=[],
-                                                  lineno=lineno, col=col),
-                                       xonsh_call('locals', args=[],
-                                                  lineno=lineno, col=col)],
-                                 lineno=lineno, col=col),
-                  cause=None, lineno=lineno, col_offset=col)],
-                orelse=[], lineno=lineno, col_offset=col)
-        node.body.insert(0, check)
-
-    def _find_with_block_line_idx(self, node):
-        ldx = min_line(node.body[0]) - 1
-        udx = max_line(node.body[-1])
-        # now check if parsable, or add lines until it is or we run out of lines
-        nlines = len(self.lines)
-        lines = 'with __xonsh_dummy__:\n' + '\n'.join(self.lines[ldx:udx])
-        lines += '\n'
-        parsable = False
-        while not parsable and udx < nlines:
-            try:
-                self.parser.parse(lines, mode=self.mode)
-                parsable = True
-            except SyntaxError:
-                lines += self.lines[udx] + '\n'
-                udx += 1
-        return ldx, udx
-
     #
     # Replacement visitors
     #
@@ -440,7 +397,6 @@ class CtxAwareTransformer(NodeTransformer):
         self._nwith += 1
         self.generic_visit(node)
         self._nwith -= 1
-        self.insert_with_block_check(node)
         return node
 
     def visit_For(self, node):
