@@ -217,7 +217,7 @@ class CommandsCache(cabc.Mapping):
     def default_predictor(self, name, cmd0):
         if ON_POSIX:
             return self.default_predictor_readbin(name, cmd0,
-                                                  timeout=.1,
+                                                  timeout=0.1,
                                                   failure=predict_true)
         else:
             return predict_true
@@ -353,6 +353,8 @@ def default_threadable_predictors():
         'mvim': predict_help_ver,
         'mutt': predict_help_ver,
         'nano': predict_help_ver,
+        'nvim': predict_false,
+        'ponysay': predict_help_ver,
         'psql': predict_false,
         'python': predict_shell,
         'python2': predict_shell,
@@ -375,6 +377,7 @@ def default_threadable_predictors():
         'weechat': predict_help_ver,
         'xo': predict_help_ver,
         'xonsh': predict_shell,
+        'xon.sh': predict_shell,
         'zsh': predict_shell,
     }
     return predictors
