@@ -8,7 +8,8 @@ import collections.abc as cabc
 
 from xonsh.ast import CtxAwareTransformer
 from xonsh.parser import Parser
-from xonsh.tools import subproc_toks, find_next_break
+from xonsh.tools import (subproc_toks, find_next_break, get_logical_line,
+                         replace_logical_line)
 from xonsh.built_ins import load_builtins, unload_builtins
 
 
@@ -89,7 +90,8 @@ class Execer(object):
             ctx = set()
         elif isinstance(ctx, cabc.Mapping):
             ctx = set(ctx.keys())
-        tree = self.ctxtransformer.ctxvisit(tree, input, ctx, mode=mode)
+        tree = self.ctxtransformer.ctxvisit(tree, input, ctx, mode=mode,
+                                            debug_level=self.debug_level)
         return tree
 
     def compile(self, input, mode='exec', glbs=None, locs=None, stacklevel=2,
@@ -153,6 +155,7 @@ class Execer(object):
         last_error_line = last_error_col = -1
         parsed = False
         original_error = None
+        greedy = False
         if filename is None:
             filename = self.filename
         while not parsed:
@@ -178,14 +181,14 @@ class Execer(object):
                 last_error_line = e.loc.lineno
                 idx = last_error_line - 1
                 lines = input.splitlines()
-                line = lines[idx]
+                line, nlogical = get_logical_line(lines, idx)
                 if input.endswith('\n'):
                     lines.append('')
                 if len(line.strip()) == 0:
                     # whitespace only lines are not valid syntax in Python's
                     # interactive mode='single', who knew?! Just ignore them.
                     # this might cause actual sytax errors to have bad line
-                    # numbers reported, but should only effect interactive mode
+                    # numbers reported, but should only affect interactive mode
                     del lines[idx]
                     last_error_line = last_error_col = -1
                     input = '\n'.join(lines)
@@ -198,9 +201,16 @@ class Execer(object):
                     if prev_indent == curr_indent:
                         raise original_error
                 lexer = self.parser.lexer
-                maxcol = find_next_break(line, mincol=last_error_col,
-                                         lexer=lexer)
-                sbpline = subproc_toks(line, returnline=True,
+                maxcol = None if greedy else find_next_break(line,
+                                                             mincol=last_error_col,
+                                                             lexer=lexer)
+                if not greedy and maxcol in (e.loc.column + 1, e.loc.column):
+                    # go greedy the first time if the syntax error was because
+                    # we hit an end token out of place. This usually indicates
+                    # a subshell or maybe a macro.
+                    greedy = True
+                    maxcol = None
+                sbpline = subproc_toks(line, returnline=True, greedy=greedy,
                                        maxcol=maxcol, lexer=lexer)
                 if sbpline is None:
                     # subprocess line had no valid tokens,
@@ -210,6 +220,9 @@ class Execer(object):
                         last_error_line = last_error_col = -1
                         input = '\n'.join(lines)
                         continue
+                    elif not greedy:
+                        greedy = True
+                        continue
                     else:
                         # or for some other syntax error
                         raise original_error
@@ -218,8 +231,13 @@ class Execer(object):
                     # if we have already wrapped this in subproc tokens
                     # and it still doesn't work, adding more won't help
                     # anything
-                    raise original_error
+                    if not greedy:
+                        greedy = True
+                        continue
+                    else:
+                        raise original_error
                 else:
+                    # print some debugging info
                     if self.debug_level > 1:
                         msg = ('{0}:{1}:{2}{3} - {4}\n'
                                '{0}:{1}:{2}{3} + {5}')
@@ -227,7 +245,8 @@ class Execer(object):
                         msg = msg.format(self.filename, last_error_line,
                                          last_error_col, mstr, line, sbpline)
                         print(msg, file=sys.stderr)
-                    lines[idx] = sbpline
+                    # replace the line
+                    replace_logical_line(lines, sbpline, idx, nlogical)
                 last_error_col += 3
                 input = '\n'.join(lines)
         return tree, input
