@@ -14,7 +14,7 @@ import collections.abc as cabc
 
 from xonsh.lazyasd import lazyobject
 from xonsh.tools import to_bool, ensure_string
-from xonsh.platform import ON_WINDOWS, ON_CYGWIN
+from xonsh.platform import ON_WINDOWS, ON_CYGWIN, os_environ
 
 COMMAND = """{seterrprevcmd}
 {prevcmd}
@@ -65,11 +65,19 @@ fi
 echo $namefile"""
 
 DEFAULT_ZSH_FUNCSCMD = """# get function names
+autoload -U is-at-least  # We'll need to version check zsh
 namefile="{"
 for name in ${(ok)functions}; do
+  # force zsh to load the func in order to get the filename,
+  # but use +X so that it isn't executed.
+  autoload +X $name || continue
   loc=$(whence -v $name)
   loc=${(z)loc}
-  file=${loc[7,-1]}
+  if is-at-least 5.2; then
+    file=${loc[-1]}
+  else
+    file=${loc[7,-1]}
+  fi
   namefile="${namefile}\\"${name}\\":\\"${(Q)file:A}\\","
 done
 if [[ "{" == "${namefile}" ]]; then
@@ -171,7 +179,8 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
                        aliascmd=None, extra_args=(), currenv=None,
                        safe=True, prevcmd='', postcmd='', funcscmd=None,
                        sourcer=None, use_tmpfile=False, tmpfile_ext=None,
-                       runcmd=None, seterrprevcmd=None, seterrpostcmd=None):
+                       runcmd=None, seterrprevcmd=None, seterrpostcmd=None,
+                       show=False, dryrun=False):
     """Extracts data from a foreign (non-xonsh) shells. Currently this gets
     the environment, aliases, and functions but may be extended in the future.
 
@@ -228,14 +237,19 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
         of the script. For example, this is "if errorlevel 1 exit 1" in
         cmd.exe. To disable exit-on-error behavior, simply pass in an
         empty string.
+    show : bool, optional
+        Whether or not to display the script that will be run.
+    dryrun : bool, optional
+        Whether or not to actually run and process the command.
+
 
     Returns
     -------
     env : dict
-        Dictionary of shell's environment
+        Dictionary of shell's environment. (None if the subproc command fails)
     aliases : dict
         Dictionary of shell's alaiases, this includes foreign function
-        wrappers.
+        wrappers.(None if the subproc command fails)
     """
     cmd = [shell]
     cmd.extend(extra_args)  # needs to come here for GNU long options
@@ -257,6 +271,10 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
                              postcmd=postcmd, funcscmd=funcscmd,
                              seterrprevcmd=seterrprevcmd,
                              seterrpostcmd=seterrpostcmd).strip()
+    if show:
+        print(command)
+    if dryrun:
+        return None, None
     cmd.append(runcmd)
     if not use_tmpfile:
         cmd.append(command)
@@ -284,7 +302,7 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
             os.remove(tmpfile.name)
     env = parse_env(s)
     aliases = parse_aliases(s)
-    funcs = parse_funcs(s, shell=shell, sourcer=sourcer)
+    funcs = parse_funcs(s, shell=shell, sourcer=sourcer, extra_args=extra_args)
     aliases.update(funcs)
     return env, aliases
 
@@ -352,7 +370,7 @@ def FUNCS_RE():
                       flags=re.DOTALL)
 
 
-def parse_funcs(s, shell, sourcer=None):
+def parse_funcs(s, shell, sourcer=None, extra_args=()):
     """Parses the funcs portion of a string into a dict of callable foreign
     function wrappers.
     """
@@ -382,7 +400,8 @@ def parse_funcs(s, shell, sourcer=None):
         if not os.path.isabs(filename):
             filename = os.path.abspath(filename)
         wrapper = ForeignShellFunctionAlias(name=funcname, shell=shell,
-                                            sourcer=sourcer, filename=filename)
+                                            sourcer=sourcer, filename=filename,
+                                            extra_args=extra_args)
         funcs[funcname] = wrapper
     return funcs
 
@@ -395,7 +414,7 @@ class ForeignShellFunctionAlias(object):
     INPUT = ('{sourcer} "{filename}"\n'
              '{funcname} {args}\n')
 
-    def __init__(self, name, shell, filename, sourcer=None):
+    def __init__(self, name, shell, filename, sourcer=None, extra_args=()):
         """
         Parameters
         ----------
@@ -407,6 +426,8 @@ class ForeignShellFunctionAlias(object):
             Where the function is defined, path to source.
         sourcer : str or None, optional
             Command to source foreing files with.
+        extra_args : tuple of str, optional
+            Addtional command line options to pass into the shell.
         """
         sourcer = DEFAULT_SOURCERS.get(shell, 'source') if sourcer is None \
             else sourcer
@@ -414,19 +435,23 @@ class ForeignShellFunctionAlias(object):
         self.shell = shell
         self.filename = filename
         self.sourcer = sourcer
+        self.extra_args = extra_args
 
     def __eq__(self, other):
         if not hasattr(other, 'name') or not hasattr(other, 'shell') or \
-                not hasattr(other, 'filename') or not hasattr(other, 'sourcer'):
+                not hasattr(other, 'filename') or not hasattr(other, 'sourcer') \
+                or not hasattr(other, 'exta_args'):
             return NotImplemented
         return (self.name == other.name) and (self.shell == other.shell) and \
-               (self.filename == other.filename) and (self.sourcer == other.sourcer)
+               (self.filename == other.filename) and \
+               (self.sourcer == other.sourcer) and \
+               (self.extra_args == other.extra_args)
 
     def __call__(self, args, stdin=None):
         args, streaming = self._is_streaming(args)
         input = self.INPUT.format(sourcer=self.sourcer, filename=self.filename,
                                   funcname=self.name, args=' '.join(args))
-        cmd = [self.shell, '-c', input]
+        cmd = [self.shell] + list(self.extra_args) + ['-c', input]
         env = builtins.__xonsh_env__
         denv = env.detype()
         if streaming:
@@ -514,7 +539,7 @@ def _get_shells(shells=None, config=None, issue_warning=True):
     elif shells is not None:
         pass
     else:
-        env = getattr(builtins, '__xonsh_env__', os.environ)
+        env = getattr(builtins, '__xonsh_env__', os_environ)
         if env.get('LOADED_CONFIG', False):
             conf = builtins.__xonsh_config__
         else:
@@ -580,11 +605,14 @@ def load_foreign_aliases(shells=None, config=None, issue_warning=True):
     for shell in shells:
         shell = ensure_shell(shell)
         _, shaliases = foreign_shell_data(**shell)
-        for alias in set(shaliases) & set(xonsh_aliases):
-            del shaliases[alias]
-            print('aliases: alias {!r} of shell {!r} '
-                  'tries to override xonsh alias, '
-                  'xonsh wins!'.format(alias, shell['shell']),
-                  file=sys.stderr)
+        if not builtins.__xonsh_env__.get('FOREIGN_ALIASES_OVERRIDE'):
+            shaliases = {} if shaliases is None else shaliases
+            for alias in set(shaliases) & set(xonsh_aliases):
+                del shaliases[alias]
+                if builtins.__xonsh_env__.get('XONSH_DEBUG') > 1:
+                    print('aliases: ignoring alias {!r} of shell {!r} '
+                          'which tries to override xonsh alias.'
+                          ''.format(alias, shell['shell']),
+                          file=sys.stderr)
         aliases.update(shaliases)
     return aliases
