@@ -10,7 +10,7 @@ import subprocess
 import collections
 
 from xonsh.lazyasd import LazyObject
-from xonsh.platform import ON_DARWIN, ON_WINDOWS, ON_CYGWIN, LIBC
+from xonsh.platform import FD_STDERR, ON_DARWIN, ON_WINDOWS, ON_CYGWIN, LIBC
 from xonsh.tools import unthreadable
 
 
@@ -127,8 +127,7 @@ else:
             LIBC.sigprocmask(ctypes.c_int(signal.SIG_BLOCK),
                              ctypes.byref(mask),
                              ctypes.byref(omask))
-            st = sys.stderr.fileno()
-            LIBC.tcsetpgrp(ctypes.c_int(st), ctypes.c_int(pgid))
+            LIBC.tcsetpgrp(ctypes.c_int(FD_STDERR), ctypes.c_int(pgid))
             LIBC.sigprocmask(ctypes.c_int(signal.SIG_SETMASK),
                              ctypes.byref(omask), None)
             return True
@@ -136,9 +135,24 @@ else:
         def give_terminal_to(pgid):
             oldmask = signal.pthread_sigmask(signal.SIG_BLOCK,
                                              _block_when_giving)
-            os.tcsetpgrp(sys.stderr.fileno(), pgid)
-            signal.pthread_sigmask(signal.SIG_SETMASK, oldmask)
-            return True
+            try:
+                os.tcsetpgrp(FD_STDERR, pgid)
+                return True
+            except OSError as e:
+                if e.errno == 22:  # [Errno 22] Invalid argument
+                    # there are cases that all the processes of pgid have
+                    # finished, then we don't need to do anything here, see
+                    # issue #2220
+                    return False
+                elif e.errno == 25:  # [Errno 25] Inappropriate ioctl for device
+                    # There are also cases where we are not connected to a
+                    # real TTY, even though we may be run in interactive
+                    # mode. See issue #2267 for an example with emacs
+                    return False
+                else:
+                    raise
+            finally:
+                signal.pthread_sigmask(signal.SIG_SETMASK, oldmask)
 
     def wait_for_active_job(last_task=None, backgrounded=False):
         """
@@ -235,7 +249,7 @@ def add_job(info):
     info['status'] = "running"
     tasks.appendleft(num)
     builtins.__xonsh_all_jobs__[num] = info
-    if info['bg']:
+    if info['bg'] and builtins.__xonsh_env__.get('XONSH_INTERACTIVE'):
         print_one_job(num)
 
 
@@ -349,7 +363,8 @@ def fg(args, stdin=None):
     job = get_task(tid)
     job['bg'] = False
     job['status'] = "running"
-    print_one_job(tid)
+    if builtins.__xonsh_env__.get('XONSH_INTERACTIVE'):
+        print_one_job(tid)
     pipeline = job['pipeline']
     pipeline.resume(job)
 
