@@ -14,6 +14,11 @@ import subprocess
 import importlib.util
 
 from xonsh.lazyasd import LazyBool, lazyobject, lazybool
+# do not import any xonsh-modules here to avoid circular dependencies
+
+FD_STDIN = 0
+FD_STDOUT = 1
+FD_STDERR = 2
 
 
 @lazyobject
@@ -27,9 +32,6 @@ def distro():
     return d
 
 
-# do not import any xonsh-modules here to avoid circular dependencies
-
-
 #
 # OS
 #
@@ -72,6 +74,14 @@ def ON_BEOS():
 
 PYTHON_VERSION_INFO = sys.version_info[:3]
 """ Version of Python interpreter as three-value tuple. """
+
+
+@lazyobject
+def PYTHON_VERSION_INFO_BYTES():
+    """The python version info tuple in a canonical bytes form."""
+    return '.'.join(map(str, sys.version_info)).encode()
+
+
 ON_ANACONDA = LazyBool(
     lambda: any(s in sys.version for s in {'Anaconda', 'Continuum', 'conda-forge'}),
     globals(), 'ON_ANACONDA')
@@ -176,6 +186,30 @@ def pathbasename(p):
     """
     return pathsplit(p)[-1]
 
+
+@lazyobject
+def expanduser():
+    """Dispatches to the correct platform-dependent expanduser() function."""
+    if ON_WINDOWS:
+        return windows_expanduser
+    else:
+        return os.path.expanduser
+
+
+def windows_expanduser(path):
+    """A Windows-specific expanduser() function for xonsh. This is needed
+    since os.path.expanduser() does not check on Windows if the user actually
+    exists. This restircts expanding the '~' if it is not followed by a
+    separator. That is only '~/' and '~\' are expanded.
+    """
+    if not path.startswith('~'):
+        return path
+    elif len(path) < 2 or path[1] in seps:
+        return os.path.expanduser(path)
+    else:
+        return path
+
+
 # termios tc(get|set)attr indexes.
 IFLAG = 0
 OFLAG = 1
@@ -296,22 +330,28 @@ def windows_bash_command():
     # Check that bash is on path otherwise try the default directory
     # used by Git for windows
     wbc = 'bash'
-    bash_on_path = builtins.__xonsh_commands_cache__.lazy_locate_binary('bash',
-                                                                        ignore_alias=True)
+    cmd_cache = builtins.__xonsh_commands_cache__
+    bash_on_path = cmd_cache.lazy_locate_binary('bash', ignore_alias=True)
     if bash_on_path:
-        # Check if Bash is from the "Windows Subsystem for Linux" (WSL)
-        # which can't be used by xonsh foreign-shell/completer
-        out = subprocess.check_output([bash_on_path, '--version'],
-                                      stderr=subprocess.PIPE,
-                                      universal_newlines=True)
-        if 'pc-linux-gnu' in out.splitlines()[0]:
+        try:
+            out = subprocess.check_output([bash_on_path, '--version'],
+                                          stderr=subprocess.PIPE,
+                                          universal_newlines=True)
+        except subprocess.CalledProcessError:
+            bash_works = False
+        else:
+            # Check if Bash is from the "Windows Subsystem for Linux" (WSL)
+            # which can't be used by xonsh foreign-shell/completer
+            bash_works = 'pc-linux-gnu' not in out.splitlines()[0]
+
+        if bash_works:
+            wbc = bash_on_path
+        else:
             gfwp = git_for_windows_path()
             if gfwp:
                 bashcmd = os.path.join(gfwp, 'bin\\bash.exe')
                 if os.path.isfile(bashcmd):
                     wbc = bashcmd
-        else:
-            wbc = bash_on_path
     return wbc
 
 #
@@ -319,6 +359,20 @@ def windows_bash_command():
 #
 
 
+@lazyobject
+def os_environ():
+    """This dispatches to the correct, case-senstive version of os.envrion.
+    This is mainly a problem for Windows. See #2024 for more details.
+    This can probably go away once support for Python v3.5 or v3.6 is
+    dropped.
+    """
+    if ON_WINDOWS:
+        import nt
+        return nt.environ
+    else:
+        return os.environ
+
+
 @functools.lru_cache(1)
 def bash_command():
     """Determines the command for Bash on the current plaform."""
